//******************************************************************************************************************
// talking_dtmf_relay.ino
// Read DTMF codes from MT8870 module and activate a relay + voice confirmation.  George Thomas, W5JDX 10/04/2018
// ** Version 2 updated 12/20/2018 to correct for DTMF module wiring **
// Password protected with timeout.
// Read and speak temperature from LM34 temperature sensor. Warning for upper and lower limits.
// Read and speak power supply voltage. Warning for upper and lower limits.
// See schematic Talking_dtmf_relay_layout.png
// Requires:
//          MT8870 DTMF Decoder board
//          LM34 Temperature Sensor
//          Talkie library from: https://github.com/going-digital/Talkie
//          misc. components
//******************************************************************************************************************

// -- establish pin usage --
int STQ = 8;                                      // DTMF Module STQ Pin connects to Arduino Digital Pin 8
int Q4 = 9;                                       // DTMF Module Q4  Pin connects to Arduino Digital Pin 9
int Q3 = 10;                                      // DTMF Module Q3  Pin connects to Arduino Digital Pin 10
int Q2 = 11;                                      // DTMF Module Q2  Pin connects to Arduino Digital Pin 11
int Q1 = 12;                                      // DTMF Module Q1  Pin connects to Arduino Digital Pin12

int controlledRelayPin = 6;                       // Controlled relay driver connects to Arduino Digital Pin 6
int pttRelayPin = 7;                              // PTT relay driver connects to Arduino Digital Pin 7

int voltPin = A0;                                 //input pin for voltage monitoring
int tempPin = A1;                                 //input pin from LM34 temperature sensor

// -- voice init --
#include "talkie.h"                               //stored in your libraries folder
#include "words.h"                                //words used in this program
Talkie voice;                                     //create a Talkie object, name it voice

// -- timing & measurement parameters --
bool receiving = false;                           //true during DTMF tone
unsigned long currentMillis = 0;                  //current time in ms since program booted
unsigned long previousStatusCheck = 0;            //last time temperature and voltage were automatically checked
const long statusInterval = 60000;                //time between status checks (1 minute)
unsigned long previousMillis = 0;                 //last read millis clock value
unsigned long previousAuthenticationMillis = 0;   //millis at last authenticated keypress
const long keyWaitInterval = 5000;                //interval to wait for single key presses (5 seconds)
const long authenticationInterval = 30000;        //length to allow commands (30 seconds)
bool authenticated = false;                       //password is currently authenticated
int passKeys[4];                                  //array to hold 3 password keys. 0 is empty
int passKeyPointer = 0;                           //pointer to last pass key stored
const int password[4] = {0, 7, 8, 9};             //1st digit always 0, operator only enters last 3
float temperature = 0;                            //current temperature
const float minTemp = 60;                         //lower temperature warning level
const float maxTemp = 100;                        //upper temperature warning level
float volts = 0;                                  //current voltage
const float minVolts = 11;                        //lower voltage warning level
const float maxVolts = 14;                        //upper voltage warning level
int warningRepeats = 0;                           //counter of current warning repeats
const int maxWarningRepeats = 3;                  //number of times to repeat a warning message before delaying
long lastWarningTime = 0;                         //time of warning at maxWarningRepeats count
const long warningInterval = 30000;               //length of ms to delay after warningRepeats number of time

void setup()                    // ---------- initialize the Arduino ----------
{
  pinMode(STQ, INPUT);              //set Digital I/O pin functions
  pinMode(Q4, INPUT);
  pinMode(Q3, INPUT);
  pinMode(Q2, INPUT);
  pinMode(Q1, INPUT);
  pinMode(controlledRelayPin, OUTPUT);
  pinMode(pttRelayPin, OUTPUT);
  Serial.begin(9600);               //open port to write val to serial port for debugging
  Serial.println("Begin dtmf_relay Program");
  saySignOn();                      //broadcast sign on message
}

void loop()                     // ---------- main loop of program ----------
{  
  currentMillis = millis();         //current time
  if (digitalRead(STQ) == HIGH)     //If STQ is High a DTMF tone is present. Check 4-bit code.
  {
    if (receiving != true){         //same keypress active, ignore until STQ goes low
      receiving = true;             //flag a key was pressed
      getData();                    //call function to get the key pressed
    }
  }
  else
  {
    receiving = false;              //reset receive flag
  }

  checkVolts();                     //check Voltage range and alert as necessary
  checkTemperature();               //check temperature range and alert as necessary
}

void getData()                  // ---------- read DTMF codes ----------
{
    byte DTMFdata = 0;              //Initialize variables
    int i = 0;
    
    if (digitalRead(Q1) == HIGH) {  //If Q1 High add 1 to DTMFdata
      DTMFdata = DTMFdata + 1;
    }
    if (digitalRead(Q2) == HIGH) {  //If Q2 High add 2 to DTMFdata
      DTMFdata = DTMFdata + 2;
    }
    if (digitalRead(Q3) == HIGH) {  //If Q3 High add 4 to DTMFdata
      DTMFdata = DTMFdata + 4;
    }
    if (digitalRead(Q4) == HIGH) {  //If Q4 High add 8 to DTMFdata
      DTMFdata = DTMFdata + 8;
    }

    Serial.print("DTMF Code = ");   //print code to screen for debugging
    Serial.println(DTMFdata);

    authenticate(DTMFdata);         //authenticate the request
    if(authenticated == true)
    {
      performCommand(DTMFdata);     //perform the command
    }
}

